/** @file
  This sample application bases on TraceDemo PCD setting 
  to print "UEFI Hello World!" to the UEFI Console.

  Copyright (c) 2006 - 2008, Intel Corporation. All rights reserved.<BR>
  This program and the accompanying materials                          
  are licensed and made available under the terms and conditions of the BSD License         
  which accompanies this distribution.  The full text of the license may be found at        
  http://opensource.org/licenses/bsd-license.php                                            

  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,                     
  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.             

**/

#include <Base.h>
#include <Uefi.h>
#include <Library/BaseLib.h>
#include <Library/DebugLib.h>
#include <Library/PcdLib.h>
#include <Library/UefiLib.h>
#include <Library/UefiApplicationEntryPoint.h>
#include <Library/DebugLibSystemTrace.h>
#include <Library/IoLib.h>
#include <Library/DebugLib.h>
#include <Library/CmosAccessLib.h>
//#include <Library/BasePlatformCmosLib.h>
#include <Library/PlatformCmosAccessLib.h>
#include "TraceDemo.h"

#pragma warning ( disable : 4189 )
#pragma warning ( disable : 4305 )
#pragma warning ( disable : 4306 )
#pragma warning ( disable : 4804 )
#pragma warning ( disable : 4244 )
#pragma warning ( disable : 4311 )
#pragma warning ( disable : 4312 )

/**
  The user Entry Point for Application. The user code starts with this function
  as the real entry point for the application.

  @param[in] ImageHandle    The firmware allocated handle for the EFI image.  
  @param[in] SystemTable    A pointer to the EFI System Table.
  
  @retval EFI_SUCCESS       The entry point is executed successfully.
  @retval other             Some error occurs when executing this entry point.

**/

#define ENTRY L"TraceHub Demo Entry.\n"
#define EXIT L"TraceHub Demo Exit.\n"

EFI_STATUS
EFIAPI
UefiMain (
  IN EFI_HANDLE        ImageHandle,
  IN EFI_SYSTEM_TABLE  *SystemTable
  )
{
	CHAR16* MYENTRY = ENTRY;
	CHAR16* MYEXIT = EXIT;
	UINT16 index;
	UINT16 nearsqrt1[MAX_ENTRIES];
	UINT16 nearsqrt2[MAX_ENTRIES];
	UINT32 sqrt1;
	UINT32 sqrt2;
	UINT32 cmos_sqrt1;
	UINT32 cmos_sqrt2;
	UINT8 cmos_block[256];

	//RETURN_STATUS status;
	UINT32 values[] = {0x29, 0x2F, 0x34, 0x4C, 0x55, 0x7e, 0x80, 0x9A, 0xC2, 0xD7, 0xEE};

    //
    // Three PCD type (FeatureFlag, UINT32 and String) are used as the sample.
    //
    Print ((CHAR16*)MYENTRY);

    //if (FeaturePcdGet (PcdTraceDemoPrintEnable)) {
  	
		// populate the cmos entries
		for (index = 0; index < 0x100; index++) {
			ReadCmosLocal((UINT8)index, &cmos_block[index]);
		}

  		for (index = 0; index < sizeof(values)/sizeof(values[0]); index ++) {

			// update the host structure with the next value
			sqrt1 = ReadCmos8(values[index]);
			cmos_sqrt1 = cmos_block[values[index]];

			sqrt2 = ReadCmos8(values[index] + 1);
			cmos_sqrt2 = cmos_block[values[index]+1];

			// perform the square root test
			nearsqrt1[index] = (UINT16)calcNearSqrt(sqrt1);
			nearsqrt2[index] = (UINT16)calcNearSqrt(sqrt2);
					
			DEBUG_GPRS();

			hPrintf2(0x75, 0x22, "index 0x%x, 0x%x\n", values[index], (values[index] + 1));
			hPrintf4(0x75, 0x22, " - using values 0x%x [%d] and 0x%x [%d]\n", cmos_sqrt1, cmos_sqrt1, cmos_sqrt2, cmos_sqrt2);
			hPrintf2(0x75, 0x22, " -- nearest square integer of %d = %u\n", sqrt1, nearsqrt1[index]);
			hPrintf2(0x75, 0x22, " -- nearest square integer of %d = %u\n", sqrt2, nearsqrt2[index]);

		}

    //}
    Print ((CHAR16*)MYEXIT);
	
	return EFI_SUCCESS;
}

/**
  Returns the nearest integer from the square root of value.
  
  @param[in]  Value			 Value to calculate the Square Root of

  @retval     Contains the nearest Integer Square of "value"
**/
UINT8
calcNearSqrt (
	IN UINT8 value
	)
{
	UINT8 guess = 2;
	UINT8 lastguess = 0;
	UINT8 div;

	while ((UINT8)guess != (UINT8)lastguess) {
		lastguess = guess;

		div = value / guess;
		guess = ((div + guess)/2);
	}
	return (UINT8)guess;
}

/**
  Returns the value from a CMOS location.

  If the passed address is beyond the max address return RETURN_NOT_FOUND.
  If the function completed successfully return RETURN_SUCCESS
  
  @param[in]  CmosAddress    Location to read from CMOS
  @param[out] Data           Contains the value read from the CMOS

  @retval     RETURN_SUCCESS
  @retval     RETURN_NOT_FOUND
**/
UINT32
ReadCmosLocal (
  IN  UINT8 CmosAddress,
  OUT UINT8 *Data
  )
{
  //
  // Initial validation of the CMOS address
  // 
  if ((CmosAddress < CMOS_START_ADDR) || (CmosAddress > CMOS_END_ADDR) || 
      (CmosAddress == CMOS_CHECKSUM_ADDR_LOW) || (CmosAddress == CMOS_CHECKSUM_ADDR_HIGH)) {
    return RETURN_NOT_FOUND;
  }

  if ((CmosAddress >= CMOS_LOW_MEM_ST) && (CmosAddress <= CMOS_LOW_MEM_END)) {
    WriteIoPortTrace (PORT_70, CmosAddress);
    *Data = (UINT8)ReadIoPortTrace (PORT_71);
  } else if ((CmosAddress >= CMOS_HIGH_MEM_ST) && (CmosAddress <= CMOS_HIGH_MEM_END)) {
    WriteIoPortTrace (PORT_72, CmosAddress);
    *Data = (UINT8)ReadIoPortTrace (PORT_73);
  } else {
    return RETURN_NOT_FOUND;
  }

  return RETURN_SUCCESS;
}

/**
  Writes an 8-bit to I/O port.

  Writes the 8-bit to I/O port specified by PortAdd with the value specified by Value
  and returns Value.

  If 8-bit I/O port operations are not supported, then ASSERT().

  @param[in]  PortAdd  The I/O port to write.
  @param[in]  Value    The value to write to the I/O port.

  @retval     Value    The value written to the I/O port.
**/
UINT8
WriteIoPortTrace (
  IN UINT8 PortAdd,
  IN UINT8 Value
  )
{
  //hPrintf2(0x77, 0x21, "WriteIO Port 0x%x with 0x%x\n", (UINT8)PortVal, (UINT8)Value);
  return IoWrite8 (PortAdd, Value);
}


/**
  Reads an 8-bit from I/O port.

  Reads the 8-bit from I/O port specified by PortAdd and returns value.

  If 8-bit I/O port operations are not supported, then ASSERT().

  @param[in]  PortAdd  The I/O port to read.

  @retval     Value    The value read from the I/O port.
**/
UINT8
ReadIoPortTrace (
  IN UINT8 PortAdd
  )
{
  return IoRead8 (PortAdd);
}

/**
  Read a byte value from a CMOS address.

  @param [in] CmosAddress   Location to read from CMOS

  @return The byte value read from the CMOS address.
**/
UINT8
EFIAPI
ReadCmos8 (
  IN  UINT8 CmosAddress
  )
{
	UINT8 Value;
	ReadCmosLocal(CmosAddress, &Value);
	return Value;
}
